# passerelle - uniform access to multiple data sources and services
# Copyright (C) 2022  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json

from django.db import models
from django.utils.translation import ugettext_lazy as _

from passerelle.sms.models import SMSResource
from passerelle.utils.jsonresponse import APIError

GSM_CHARACTERS = set(
    '@£$¥èéùìòÇØøÅåΔ_ΦΓΛΩΠΨΣΘΞ^{}[~]|€ÆæßÉ!»#¤%&‘()*+,./0123456789:;<=>?¡ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÑÜ§¿abcdefghijklmnopqrstuvwxyzäöñüà. '
)


class SfrDmcGateway(SMSResource):
    service_id = models.CharField(verbose_name=_('Service ID'), max_length=15)
    service_password = models.CharField(verbose_name=_('Service Password'), max_length=15)
    space_id = models.IntegerField(verbose_name=_('Space ID'))

    TEST_DEFAULTS = {
        'create_kwargs': {
            'service_id': '1234',
            'service_password': 'krascuky',
            'space_id': '1234',
        },
        'test_vectors': [
            {
                'response': '',
                'result': {
                    'err': 1,
                    'err_desc': 'Bad JSON response',
                    'data': None,
                },
            },
            {
                'response': {'bad_json': True},
                'result': {
                    'err': 1,
                    'err_desc': 'Bad JSON response',
                    'data': None,
                },
            },
            {
                'response': {'success': False, 'no_error_detail_field': 'bad_json'},
                'result': {'err': 1, 'err_desc': 'Bad JSON response', 'data': None},
            },
            {
                'response': {'success': False, 'errorDetail': 'sfr error description'},
                'result': {
                    'err': 1,
                    'err_desc': 'sfr error description',
                    'data': None,
                },
            },
            {
                'response': {'success': True, 'response': ['1234', '5678']},
                'result': {
                    'err': 0,
                    'data': [
                        ['0033688888888', {'result': 'Envoi terminé', 'sms_id': 1234}],
                        ['0033677777777', {'result': 'Envoi terminé', 'sms_id': 1234}],
                    ],
                },
            },
        ],
    }
    URL = 'https://www.dmc.sfr-sh.fr/DmcWS/1.5.7'

    class Meta:
        verbose_name = _('SFR DMC')

    def send_msg(self, text, sender, destinations, **kwargs):
        """Send a SMS using the SFR DMC provider"""

        if len(destinations) > 20:
            raise APIError('you can\'t send more than 20 sms at once.')

        sfr_format_destinations = []
        for dest in destinations:
            sfr_format_destinations.append(f'+{dest[2:]}')  # SFR API only accepts '+' prefix

        non_standard_characters = set(text) - GSM_CHARACTERS

        if len(non_standard_characters) == 0:
            media_type = 'SMSLong'
        else:
            media_type = 'SMSUnicodeLong'

        response = self.requests.get(
            f'{self.URL}/JsonService/MessagesUnitairesWS/batchSingleCall',
            params={
                'authenticate': json.dumps(
                    {
                        'serviceId': self.service_id,
                        'servicePassword': self.service_password,
                        'spaceId': self.space_id,
                    }
                ),
                'messageUnitaires': json.dumps(
                    [
                        {'media': media_type, 'textMsg': text, 'from': sender, 'to': dest}
                        for dest in sfr_format_destinations
                    ]
                ),
            },
        )

        try:
            json_data = response.json()
        except ValueError:
            raise APIError('Bad JSON response')

        if 'success' not in json_data:
            raise APIError('Bad JSON response')
        elif not json_data['success']:
            if not 'errorDetail' in json_data:
                raise APIError('Bad JSON response')
            else:
                raise APIError(json_data['errorDetail'])
